#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "connect_signals.h"
#include <libdc1394/dc1394_control.h>
#include <libraw1394/raw1394.h>
#include "video1394.h"

extern GtkWidget *commander_window;
extern GtkWidget *porthole_window;
extern GtkWidget *color_window;
extern GtkWidget *status_window;
extern GtkWidget *aperture_window;
extern GtkWidget *file_selector_window;
extern GtkWidget *configure_window;
extern GtkWidget *capture_window;
extern GtkWidget *temperature_window;
extern dc1394_camerainfo camera;
extern char* feature_op_list[NUM_FEATURES];
extern char* feature_list[NUM_FEATURES];
extern char* feature_frame_list[NUM_FEATURES];
extern char* feature_scale_list[NUM_FEATURES];
extern char* feature_man_list[NUM_FEATURES];
extern char* feature_auto_list[NUM_FEATURES];
extern char* feature_power_list[NUM_FEATURES];



void
ConnectRangeSignals()
{
  GtkAdjustment* adj;

  // porthole window:
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(porthole_window,"zoom_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_zoom_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(porthole_window,"pan_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_pan_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(porthole_window,"tilt_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_tilt_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(porthole_window,"focus_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_focus_scale_value_changed), NULL);

  // color window:
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"brightness_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_brightness_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"gamma_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_gamma_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"saturation_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_saturation_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"hue_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_hue_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"sharpness_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_sharpness_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"whitebal_BU_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_BU_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"whitebal_RV_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_RV_scale_value_changed), NULL);

  // aperture_window:
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(aperture_window,"exposure_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_exposure_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(aperture_window,"iris_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_iris_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(aperture_window,"shutter_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_shutter_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(aperture_window,"gain_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_gain_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(aperture_window,"filter_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_filter_scale_value_changed), NULL);

  // capture window:
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(capture_window,"capqual_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_capqual_scale_value_changed), NULL);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(capture_window,"capsize_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_capsize_scale_value_changed), NULL);

  // temperature window:
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(temperature_window,"temp_goal_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_temp_goal_scale_value_changed), NULL);

}

void
on_zoom_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_ZOOM,adj->value);
}

void
on_focus_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_FOCUS,adj->value);
}

void
on_pan_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_PAN,adj->value);
}

void
on_tilt_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_TILT,adj->value);
}

void
on_brightness_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_BRIGHTNESS,adj->value);
}

void
on_gamma_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_GAMMA,adj->value);
}

void
on_saturation_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_SATURATION,adj->value);
}

void
on_hue_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_HUE,adj->value);
}

void
on_RV_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err, RV, BU;
  err=dc1394_get_white_balance(camera.handle,camera.id,&BU, &RV);
  err=dc1394_set_white_balance(camera.handle,camera.id,BU, adj->value);
}

void
on_BU_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err, RV, BU;
  err=dc1394_get_white_balance(camera.handle,camera.id,&BU, &RV);
  err=dc1394_set_white_balance(camera.handle,camera.id,adj->value, RV);
}

void
on_sharpness_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_SHARPNESS,adj->value);
}

void
on_exposure_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_EXPOSURE,adj->value);
}

void
on_iris_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_IRIS,adj->value);
}

void
on_gain_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_GAIN,adj->value);
}

void
on_shutter_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_SHUTTER,adj->value);
}

void
on_filter_scale_value_changed             ( GtkAdjustment    *adj,
                                          gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_OPTICAL_FILTER,adj->value);
}

void
on_capqual_scale_value_changed             ( GtkAdjustment    *adj,
					     gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_CAPTURE_QUALITY,adj->value);
}

void
on_capsize_scale_value_changed             ( GtkAdjustment    *adj,
					     gpointer         user_data)
{
  int err;
  err=dc1394_set_feature_value(camera.handle,camera.id,FEATURE_CAPTURE_SIZE,adj->value);
}

void
on_temp_goal_scale_value_changed             ( GtkAdjustment    *adj,
					       gpointer         user_data)
{
  int err;
  err=dc1394_set_temperature(camera.handle,camera.id,adj->value);
}


